@@ -1,3 +1,4 @@
+#
 # -*- coding: utf-8 -*-
 """Handle Manager."""
 
@@ -34,8 +35,7 @@ class HandleManager(object):
         self._tb_watcher = None
         self._system_stats = None
 
-        # keep track of config and summary from key/val updates
-        # self._consolidated_config = dict()
+        # keep track of summary from key/val updates
         self._consolidated_summary = dict()
         self._sampled_history = dict()
 
@@ -194,6 +194,9 @@ class HandleManager(object):
     def handle_request_check_version(self, record):
         self._dispatch_record(record)
 
+    def handle_telemetry(self, record):
+        self._dispatch_record(record)
+
     def handle_request_run_start(self, record):
         run_start = record.request.run_start
         assert run_start
