@@ -1,3 +1,4 @@
+#
 from gql import Client, gql  # type: ignore
 from gql.client import RetryError  # type: ignore
 from gql.transport.requests import RequestsHTTPTransport  # type: ignore
@@ -1087,14 +1088,12 @@ class Api(object):
         )
         run = run or self.current_run_id
         assert run, "run must be specified"
+        entity = entity or self.settings("entity")
         query_result = self.gql(
-            query,
-            variable_values={
-                "name": project,
-                "run": run,
-                "entity": entity or self.settings("entity"),
-            },
+            query, variable_values={"name": project, "run": run, "entity": entity,},
         )
+        if query_result["model"] is None:
+            raise CommError("Run does not exist {}/{}/{}.".format(entity, project, run))
         files = self._flatten_edges(query_result["model"]["bucket"]["files"])
         return {file["name"]: file for file in files if file}
 
