@@ -38,6 +38,7 @@ import time
 
 import six
 import wandb
+from wandb import util
 
 from .lib.git import GitRepo
 from .lib.ipython import _get_python_type
@@ -77,6 +78,7 @@ env_settings: Dict[str, Optional[str]] = dict(
     problem=None,
     console=None,
     config_paths=None,
+    sweep_param_path=None,
     run_id=None,
     notebook_name=None,
     host=None,
@@ -86,6 +88,7 @@ env_settings: Dict[str, Optional[str]] = dict(
     ignore_globs=None,
     resume=None,
     silent=None,
+    sagemaker_disable=None,
     root_dir="WANDB_DIR",
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
@@ -106,13 +109,6 @@ def _build_inverse_map(prefix: str, d: Dict[str, str]) -> Dict[str, str]:
     return inv_map
 
 
-def _is_kaggle() -> bool:
-    return (
-        os.getenv("KAGGLE_KERNEL_RUN_TYPE") is not None
-        or "kaggle_environments" in sys.modules  # noqa: W503
-    )
-
-
 def _error_choices(value: str, choices: Set[str]) -> str:
     return "{} not in {}".format(value, ",".join(list(choices)))
 
@@ -217,6 +213,12 @@ class Settings(object):
     show_info: bool = True
     show_warnings: bool = True
     show_errors: bool = True
+    email: Optional[str] = None
+    save_code: Optional[bool] = None
+    program_relpath: Optional[str] = None
+
+    # Public attributes
+    sagemaker_disable: Optional[bool] = None
 
     # Private attributes
     __start_time: Optional[float]
@@ -276,6 +278,7 @@ class Settings(object):
         system_samples=15,
         heartbeat_seconds=30,
         config_paths=None,
+        sweep_param_path=None,
         _config_dict=None,
         # directories and files
         root_dir=None,
@@ -313,6 +316,7 @@ class Settings(object):
         username=None,
         email=None,
         docker=None,
+        sagemaker_disable: Optional[bool] = None,
         _start_time=None,
         _start_datetime=None,
         _cli_only_mode=None,  # avoid running any code specific for runs
@@ -407,7 +411,7 @@ class Settings(object):
 
     @property
     def _kaggle(self) -> bool:
-        return _is_kaggle()
+        return util._is_kaggle()
 
     @property
     def _windows(self) -> bool:
